package com.usky.dxtop.service.impl;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.usky.dxtop.common.core.domain.model.LoginUser;
import com.usky.dxtop.common.core.page.CommonPage;
import com.usky.dxtop.common.exception.CustomException;
import com.usky.dxtop.common.utils.DateUtils;
import com.usky.dxtop.common.utils.ExcelUtils;
import com.usky.dxtop.common.utils.SecurityUtils;
import com.usky.dxtop.common.utils.StringUtils;
import com.usky.dxtop.common.utils.file.ImageUtils;
import com.usky.dxtop.common.utils.sign.Base64;
import com.usky.dxtop.mapper.StaffMapper;
import com.usky.dxtop.model.*;
import com.usky.dxtop.service.ISysAsyncTaskService;
import com.usky.dxtop.service.MsgLogService;
import com.usky.dxtop.service.StaffService;
import com.usky.dxtop.service.SysFileService;
import com.usky.dxtop.service.config.rabbitmq.RabbitmqUtils;
import com.usky.dxtop.service.constant.MsgLogBusinessCode;
import com.usky.dxtop.service.emun.AsyncResultType;
import com.usky.dxtop.service.emun.RabbitmqBeenCode;
import com.usky.dxtop.service.vo.*;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.amqp.rabbit.connection.CorrelationData;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author yq
 * @since 2021-09-28
 */
@Service
public class StaffServiceImpl extends ServiceImpl<StaffMapper, Staff> implements StaffService {

    @Autowired
    private MsgLogService msgLogService;


    @Autowired
    private ISysAsyncTaskService sysAsyncTaskService;

    @Autowired
    private SysFileService sysFileService;

    @Override
    public CommonPage<Staff> page(StaffRequest staffRequest) {
        IPage<Staff> page = new Page<>(staffRequest.getCurrent(), staffRequest.getSize());
        page = baseMapper.page(page, staffRequest.getStartTime(),
                staffRequest.getEndTime(),
                staffRequest.getName(),
                staffRequest.getPhone(),
                staffRequest.getCard(),
                staffRequest.getDeptId());
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }

    @Override
    public BigDecimal getBalance(String card) {
        card = new BigDecimal(card).toString();
        Staff staff = this.one(null, null, card);
        Optional.ofNullable(staff).orElseThrow(() -> new CustomException("用户信息不存在"));
        return Optional.ofNullable(staff.getBalance()).orElse(new BigDecimal("0.00"));
    }

    @Override
    public Staff one(String id, String phone, String card) {
        if (StringUtils.isBlank(id) && StringUtils.isBlank(phone) && StringUtils.isBlank(card)){
            throw new CustomException("手机号，用户编号，卡号至少输入一个");
        }
        return baseMapper.one(id,phone,card);
    }


    /**
     * 人员消息
     * @param staff
     */
    @Override
    public void personSendMessage(Staff staff,Integer type){
        long seq = System.nanoTime();
        Map<String,Object> map = new HashMap<>();
        map.put("type",type);
        map.put("card_type",1);
        map.put("card",staff.getCardId());
        map.put("name",staff.getName());
        map.put("mob",staff.getSId());
        map.put("groupid",staff.getDeptId());
        map.put("tel",staff.getContacts());
        map.put("level_name","员工卡");
        map.put("seq",seq);
        MsgLog msgLog = new MsgLog();
        msgLog.setId(seq);
        msgLog.setBusinessId(String.format("%s%s", MsgLogBusinessCode.STAFF,staff.getSId()));
        msgLog.setExchange(RabbitmqBeenCode.PROFILE_CONSUMER.getName());
        msgLog.setRoutingKey(RabbitmqBeenCode.PROFILE_CONSUMER.getName());
        msgLog.setMsg(JSON.toJSONString(map));
        msgLogService.addOrUpdate(msgLog);
        CorrelationData correlationData = new CorrelationData(msgLog.getId().toString());
        RabbitmqUtils.getRabbitmqTemplate(RabbitmqBeenCode.PROFILE_CONSUMER).convertAndSend(msgLog.getExchange(), msgLog.getRoutingKey(), map,correlationData);
    }


    /**
     * 人脸
     * @param staff
     */
    @Override
    public void faceSendMessage(Staff staff){
        long seq = System.nanoTime();
        String s = StaffServiceImpl.replaceDomainAndPort("10.21.39.3", "19930", staff.getFaceId());
        String replace = s.replace("https", "http");
        String encode = Base64.encode(ImageUtils.getImage(replace));
        Map<String,Object> map = new HashMap<>();
        map.put("mob",staff.getSId());
        MsgLog msgLog = new MsgLog();
        msgLog.setBusinessId(String.format("%s%s", MsgLogBusinessCode.FACE,staff.getSId()));
        msgLog.setExchange(RabbitmqBeenCode.PROFILE_FACE_CONSUMER.getName());
        msgLog.setRoutingKey(RabbitmqBeenCode.PROFILE_FACE_CONSUMER.getName());
        map.put("img",staff.getFaceId());
        msgLog.setMsg(JSON.toJSONString(map));
        msgLog.setId(seq);
        msgLogService.addOrUpdate(msgLog);
        map.put("seq",seq);
        map.put("img",encode);
        CorrelationData correlationData = new CorrelationData(msgLog.getId().toString());
        RabbitmqUtils.getRabbitmqTemplate(RabbitmqBeenCode.PROFILE_FACE_CONSUMER).convertAndSend(msgLog.getExchange(), msgLog.getRoutingKey(), map,correlationData);
    }


    public static String replaceDomainAndPort(String domain,String port,String url){
        String url_bak = "";
        if(url.indexOf("//") != -1 ){
            String[] splitTemp = url.split("//");
            url_bak = splitTemp[0]+"//";
            if(port != null){
                url_bak = url_bak + domain+":"+port;
            }else{
                url_bak = url_bak + domain;
            }

            if(splitTemp.length >=1 && splitTemp[1].indexOf("/") != -1){
                String[] urlTemp2 = splitTemp[1].split("/");
                if(urlTemp2.length > 1){
                    for(int i = 1;i < urlTemp2.length; i++){
                        url_bak = url_bak +"/"+urlTemp2[i];
                    }
                }
            }
        }
        return url_bak;
    }


    @Override
    public List<Staff> selectStaffByPhoneList(List<String> phoneList) {
        return baseMapper.selectListByPhones(phoneList);
    }

    @Override
    public CommonPage<ChargeAndDishVO> collectByMonth(ChargeRequest chargeRequest) {
        IPage<ChargeAndDishVO> page = new Page<>(chargeRequest.getCurrent(), chargeRequest.getSize());
        page = baseMapper.selectCollectByMonth(page,chargeRequest);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());

    }

    @Override
    public String exportChargeAndDish(ChargeRequest chargeRequest) {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        SysUser user = loginUser.getUser();
        Long taskId = sysAsyncTaskService.getId(user.getUserName(), "充值消费统计");
        if (null == taskId) {
            throw new CustomException("获取次数太多了请稍后获取");
        }
        ((StaffServiceImpl) AopContext.currentProxy()).syncChargeAndDish(taskId,chargeRequest);
        return String.valueOf(taskId);
    }


    @Async
    public void syncChargeAndDish(Long taskId, ChargeRequest chargeRequest){
        Workbook workbook = null;
        try {
            ExportParams params = new ExportParams(null, "充值消费统计");
            workbook = ExcelExportUtil.exportBigExcel(params, ChargeAndDishVO.class,
                    (o, i) -> {
                        chargeRequest.setCurrent(i);
                        chargeRequest.setSize(30);
                        CommonPage<ChargeAndDishVO> page = this.collectByMonth(chargeRequest);
                        return new ArrayList<>(page.getRecords());
                    },null);
            if (null != workbook) {
                String fileName = String.format("%s.xls", "充值消费记录统计");
                MultipartFile multipartFile = ExcelUtils.downLoadExcel(workbook, fileName);
                uploadOrderFile(multipartFile,taskId);
            } else {
                sysAsyncTaskService.sendResult(taskId, false, "表格数据为空", AsyncResultType.TEXT);
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            sysAsyncTaskService.sendResult(taskId, false, "导出文件失败", AsyncResultType.TEXT);
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
    }



    public void uploadOrderFile(MultipartFile multipartFile, Long taskId){
        List<MultipartFile> list = new ArrayList<>();
        list.add(multipartFile);
        String batchNo = sysFileService.generateBatchNo();
        SysFile sysFile = new SysFile();
        sysFile.setBatchNo(batchNo);
        sysFile.setFiles(list);
        sysFile.setUploadType(SysFileServiceImpl.FILE);
        sysFile.setBusinessType("充值消费记录");
        sysFile.setExpriceAt(DateUtils.addDays(new Date(), 3));
        List<SysFile> sysFileList = sysFileService.addFile(sysFile);
        long count = sysFileList.stream().filter(sysFileDTO -> !sysFileDTO.getSuccess()).count();
        if (count > 0) {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
            return;
        }
        List<String> idList = sysFileList.stream().map(sf -> sf.getId().toString()).collect(Collectors.toList());
        boolean active = sysFileService.active(batchNo,idList);
        if (active) {
            sysAsyncTaskService.sendResult(taskId, true, batchNo, AsyncResultType.FILE);
        } else {
            sysAsyncTaskService.sendResult(taskId, false, "上传文件失败", AsyncResultType.TEXT);
        }
    }
}
